package Librerias;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.apache.poi.ss.usermodel.*;


public class LecturaEscrituraHojasCalculo {
    
   Workbook workbook;
   Sheet sheet;

   
   /*******************************************************************************************
    * Este método abre un archivo en excel, especificandole un nombre con extensión y ruta.   *
    * @param rutaArchivo: ruta donde se encuentra el archivo                                  *
    * @param nombreArchivo: nombre del archivo que se va abrir                                *
    * @param extension: extensión del archivo ya sea xls o xlsx                               *
    * @return true si se abrio correctamente el archivo de lo contrario false                 *                                         
    *******************************************************************************************/
   public boolean abrirArchivoExcel(String rutaArchivo, String nombreArchivo,String extension){

       try{
           //Se intenta abrir el archivo de excel con la ruta y nombre especificados
           workbook = extension.contains(".xlsx")? new XSSFWorkbook(new FileInputStream(rutaArchivo+nombreArchivo+extension))
                                                 : new HSSFWorkbook(new FileInputStream(rutaArchivo+nombreArchivo+extension));

       }catch (IOException e){
           return false;

       }catch (Exception e){
           e.printStackTrace();
           return false;

       }

        return true;
    }

   public boolean abrirArchivoOpenOffice(String rutaArchivo, String nombreArchivo,String extension){

       try{
           //Se intenta abrir el archivo de excel con la ruta y nombre especificados
           new FileInputStream(rutaArchivo+nombreArchivo+extension);

       }catch (IOException e){
           return false;

       }catch (Exception e){
           e.printStackTrace();
           return false;

       }

        return true;
    }


   /*******************************************************************************************
    *     Este método crea un archivo en excel nuevo en caso de no existir uno anteriormente  *
    *                          creado con el nombre y ruta especificados.                     *
    * @param rutaArchivo: ruta donde se encuentra el archivo                                  *
    * @param nombreArchivo: nombre del archivo que se va abrir                                *
    * @param extension: extensión del archivo ya sea xls o xlsx                               *                                                                   *
    * @return true si se creó correctamente el archivo de lo contrario false                  *
    *******************************************************************************************/

   public boolean crearArchivoExcel(String rutaArchivo, String nombreArchivo,String extension){
        try{
            //Se verifica que el archivo de excel existe, en caso de existir solo lo abre de 
            //lo contrario crea un nuevo archivo de excel
            if (abrirArchivoExcel(rutaArchivo, nombreArchivo,extension)){
            }else{
               workbook = extension.contains("xlsx")? new XSSFWorkbook()
                                                 : new HSSFWorkbook();
            }
       }catch (Exception e){
           e.printStackTrace();
           return false;

       }
       return true;

    }

   public boolean crearArchivoOpenOffice(String rutaArchivo, String nombreArchivo,String extension){
        try{
            //Se verifica que el archivo de excel existe, en caso de existir solo lo abre de
            //lo contrario crea un nuevo archivo de excel
            if (abrirArchivoExcel(rutaArchivo, nombreArchivo,extension)){
            }else{
               new FileInputStream(rutaArchivo+nombreArchivo+extension);
            }
       }catch (Exception e){
           e.printStackTrace();
           return false;

       }
       return true;

    }

   /***********************************************************************************************
    *          Este método crea una nueva hoja al archivo en excel                                *
    * @param nombreHoja: nombre de la hoja que se pretende leer o crear en el archivo de excel    *
    * @return true si la hoja se creó correctamente, de lo contrario false                        *
    ***********************************************************************************************/

   public boolean crearHojaExcel(String nombreHoja){

       try{
           sheet = workbook.getSheet(nombreHoja);
           if (sheet==null){
              sheet = workbook.createSheet(nombreHoja);
           }
       }catch (Exception e){
           e.printStackTrace();
           return false;

       }

        return true;
    }

   /******************************************************************************************
    *           Este método escribe en una hoja de excel previamentete                       *
    *               creada los valores almacenados en una matriz                             *
    * @param matriz: matriz que contiene los datos que se escribirán en la hoja de excel     *                                                                   *
    * @param lineaIni: linea en la que se empezarán a escribir los datos de la matriz        *
    * @param colIni: columna en la que se empezarán a escrbir los datos de la matriz         *
    * @param formato: formato que se le dará a las celdas,                                   *
    *                 en este caso numero de digitos despues del punto                       *                                                  *
    ******************************************************************************************/

   public void escribirHojasExcel(double[][] matriz, int lineaIni, int colIni,String formato){

       CellStyle cellStyle = workbook.createCellStyle();
       DataFormat format = workbook.createDataFormat();

       int fila = lineaIni - 1;
       int columna =0;

       for (double[] linea:matriz){
           Row row;
           if (fila <= sheet.getLastRowNum()){
              row = sheet.getRow(fila);
              if (row==null){
                 row = sheet.createRow(fila);
              }
           }else{
              //se crea toda la fila para poder escribir en ella
              row = sheet.createRow(fila);
           }

           //se crea toda la fila para poder escribir en ella
           //row = sheet.createRow(fila);

           columna=colIni - 1;
           for (double celda:linea){
               //se crea una celda en la fila en que nos encontramos posisionados
               Cell cell = row.createCell(columna);
               //escribe el valor de la matriz en la celda correspondiente
               cell.setCellValue(celda);
               if (!formato.equals(null)){
                   //Se da formato a la celda que indica el número de ceros despues del punto decimal
                   cellStyle.setDataFormat(format.getFormat(formato));
                   cell.setCellStyle(cellStyle);
               }
               sheet.autoSizeColumn((short)columna);
               columna++;
           }
           fila++;
       }

    }
   

   /*******************************************************************************************
    *      Este método lee el contenido en las celdas de una hoja de excel                    *
    * @param hoja: nombre de la hoja que se pretende leer o crear en el archivo de excel      *                                                             *
    * @param lineas: linea en la que se empezarán a leer los datos del archivo de excel       *
    * @param columnas: columna en la que se empezarán a leer los datos del archivo de excel   *
    * @param totalLineas: total de lineas a leer del archivo de excel                         *
    * @param totalColumnas: total de columnas a leer del archivo de excel                     *
    * @return una matriz en la que se almacenan los datos leidos del archivo de excel         *
    *******************************************************************************************/
   public double[][] leerHojaExcel(String hoja,int lineas, int columnas,int totalLineas, int totalColumnas){
        sheet = workbook.getSheet(hoja);
        Row row = null;
        Cell cell = null;
        double[][] matriz = new double [totalColumnas][totalLineas];
        for ( int i=lineas-1, z=(lineas - 1)+ (totalLineas -1); i<=z; i++ ){
            row = sheet.getRow(i);
            //verifica que el contenido de toda la fila en la que nos encontramos tenga información
            if (row != null){
            for (int ii=columnas - 1, zz=(columnas - 1)+(totalColumnas); ii<zz; ii++){
                cell = row.getCell(ii);
                //verifica que el contenido de la celda en la que nos encontramos tenga información
                if (cell != null) {
                    //en caso de que en la celda leida halla fórmula, solo se leera el valor y no la fórmula
                    if (Cell.CELL_TYPE_FORMULA == cell.getCellType()||Cell.CELL_TYPE_NUMERIC == cell.getCellType())
                        matriz[ii - (columnas - 1)][i - (lineas - 1)]= cell.getNumericCellValue();
                }
            }
          }
       }
      return matriz;
    }




    public double[] leerVectorHojaExcel(String hoja,int lineas, int columnas,int totalLineas){
        sheet = workbook.getSheet(hoja);
        Row row = null;
        Cell cell = null;
        double[] vector = new double [totalLineas];

        for ( int i=lineas-1, ii=(lineas - 1)+ (totalLineas -1); i<=ii; i++ ){
            row = sheet.getRow(i);
            //verifica que el contenido de toda la fila en la que nos encontramos tenga información
            if (row != null){
                cell = row.getCell(columnas-1);
                //verifica que el contenido de la celda en la que nos encontramos tenga información
                if (cell != null) {
                    //en caso de que en la celda leida halla fórmula, solo se leera el valor y no la fórmula
                    if (Cell.CELL_TYPE_FORMULA == cell.getCellType()||Cell.CELL_TYPE_NUMERIC == cell.getCellType())
                        vector[i - (lineas - 1)]= cell.getNumericCellValue();
                }
             }
        }

        return vector;
       }
      

   /*************************************************************************************
    * Este método guarda el contenido que se escribio previamente en la hoja de excel   *
    * @param rutaArchivoSalida: ruta en la que se almacenará el archivo de excel        *
    * @param nombreArchivoSalida: nombre del archivo de excel                           *
    * @return true si se guardó correctamente el archivo, de lo contratio falso         *
    *************************************************************************************/
   public boolean guardarLibroExcel(String rutaArchivoSalida, String nombreArchivoSalida){
        try {
            //Se asigna al archivo la extensión xls o xlsx según se halla creado el libro anteriormente
            if(workbook instanceof XSSFWorkbook) {
               nombreArchivoSalida = nombreArchivoSalida + ".xlsx";
            }else{
                nombreArchivoSalida = nombreArchivoSalida + ".xls";
            }
            FileOutputStream elFichero = new FileOutputStream(rutaArchivoSalida+nombreArchivoSalida);

            workbook.write(elFichero);
            elFichero.close();

        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
        return true;
    }

   public boolean guardarLibroOpenOffice(String rutaArchivoSalida, String nombreArchivoSalida){
        try {
            //Se asigna al archivo la extensión xls o xlsx según se halla creado el libro anteriormente
            if(workbook instanceof XSSFWorkbook) {
               nombreArchivoSalida = nombreArchivoSalida + ".ods";
            }else{
                nombreArchivoSalida = nombreArchivoSalida + ".ods";
            }
            FileOutputStream elFichero = new FileOutputStream(rutaArchivoSalida+nombreArchivoSalida);

            workbook.write(elFichero);
            elFichero.close();

        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
        return true;
    }

}
