/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package Stock;

/**
 *
 * @author Antonio-Alcalde
 */
import ProyeccionPensionados.Constantes;
import Librerias.FuncionesMatematicas;

public class ProyeccionDemograficaStock {

    private double[][] pensionadosSobrevivientes;
    private double[][] pensionadosFallecidos;
    private double[][] nuevasDerivadas;
    private double[][] derivadasSobFinAnio;
    private double[][] derivadasSobSinNuevas;
    private double[][] stockDerivadasSob;
    private double[][] casosTotalesDerivadas;

    public void obtenerPensionadosSobrevivientes(double[] casos,
                                                 double[] probMte){

        double[][] penSob = new double[Constantes.ANIOS_PROYECCION-1][(Constantes.EDAD_MAX_PROYECTADA-Constantes.EDAD_MIN_PROYECTADA)+1];

        //Para el primer año
        for(int edad=0;edad<casos.length;edad++){
            penSob[0][edad] = casos[edad];
        }        
        //Para los demas años
        for(int año = 1;año<penSob.length;año++){
            for(int edad=1;edad<penSob[0].length;edad++){
                penSob[año][edad] = penSob[año-1][edad-1] * (1-probMte[edad-1]);
            }
        }               
        this.pensionadosSobrevivientes = penSob;

    }

    public void obtenerPensionadosSobrevivientes(double[][] probSob,
                                                 double[]   casos){

        double[][] penSob = new double[Constantes.ANIOS_PROYECCION-1][(Constantes.EDAD_MAX_PROYECTADA-Constantes.EDAD_MIN_PROYECTADA)+1];

        //Para el primer año
        for(int edad=0;edad<casos.length;edad++){
            penSob[0][edad] = casos[edad];
        }
        //Para los demas años
        for(int año = 1;año<penSob.length;año++){
            for(int edad=1;edad<penSob[0].length;edad++){
                penSob[año][edad] = penSob[año-1][edad-1] * probSob[((año-1)<probSob.length?(año-1):probSob.length-1)][edad-1];
            }
        }
        this.pensionadosSobrevivientes = penSob;

    }

    public void obtenerPensionadosFallecidos(double[][] penSob,
                                             double[]   probMte){

        double[][] penFalle = new double[Constantes.ANIOS_PROYECCION-1][(Constantes.EDAD_MAX_PROYECTADA-Constantes.EDAD_MIN_PROYECTADA)+1];
        
        //Para todos los años
        for(int año=1;año<penFalle.length;año++){
            for(int edad=1;edad<penFalle[0].length;edad++){
                penFalle[año][edad] = penSob[año-1][edad-1] * probMte[edad-1];
            }
        }        
        this.pensionadosFallecidos = penFalle;


    }

    public void obtenerPensionadosFallecidos(double[][] penSob,
                                             double[][] probSob){

        double[][] penFalle = new double[Constantes.ANIOS_PROYECCION-1][(Constantes.EDAD_MAX_PROYECTADA-Constantes.EDAD_MIN_PROYECTADA)+1];

        //Para todos los años
        for(int año=1;año<penFalle.length;año++){
            for(int edad=1;edad<penFalle[0].length;edad++){
                penFalle[año][edad] = penSob[año-1][edad-1] * (1-probSob[(año-1<probSob.length?año-1:probSob.length-1)][edad-1]);
            }
        }
        this.pensionadosFallecidos = penFalle;


    }

    public void obtenerNuevasDerivadas(double[][] pensFallecidos,
                                       double[][] distDerivadas){

        double[][] nuevasDer = new double[pensFallecidos.length][pensFallecidos[0].length];

        FuncionesMatematicas fm = new FuncionesMatematicas();

        for(int año=1;año<nuevasDer.length;año++){
            for(int edad=0;edad<nuevasDer[0].length;edad++){
                nuevasDer[año][edad] = fm.sumaProducto(pensFallecidos[año], distDerivadas[edad]);
            }
        }
        this.nuevasDerivadas = nuevasDer;

    }

    public void obtenerNuevasDerivadasSobFinAnio(double[][] nuevasDerivadas,
                                                 double[][] probSobDerivadas){

        double[][] derSobFinAnio = new double[nuevasDerivadas.length][nuevasDerivadas[0].length];
        FuncionesMatematicas fm = new FuncionesMatematicas();

        for(int año=1;año<derSobFinAnio.length;año++){
            for(int edad=0;edad<derSobFinAnio[0].length;edad++){
                derSobFinAnio[año][edad] = nuevasDerivadas[año][edad]*((2*probSobDerivadas[año][edad])/(1+probSobDerivadas[año][edad]));
            }
        }
        this.derivadasSobFinAnio = derSobFinAnio;

    }

    public void obtenerDerivadasSobSinNuevas(double[][] derivadasSobFinAnio,
                                             double[][] probSobDerivadas){

        double[][] derivadasSobSinNuevas = new double[derivadasSobFinAnio.length][derivadasSobFinAnio[0].length];

        for(int año=2;año<derivadasSobSinNuevas.length;año++){
            for(int edad=1;edad<derivadasSobSinNuevas[0].length;edad++){
                derivadasSobSinNuevas[año][edad] = derivadasSobFinAnio[año-1][edad-1]* probSobDerivadas[año][edad-1];
            }
        }
        this.derivadasSobSinNuevas = derivadasSobSinNuevas;

    }

    public void obtenerStockDerivadasSob(double[][] derivadasSobSinNuevas,
                                         double[][] probSobDerivadas,
                                         double[]   stockDerivada,
                                         boolean    esCeVe){

        double[][] stockDerSob = new double[derivadasSobSinNuevas.length][derivadasSobSinNuevas[0].length];

        if(!esCeVe){
            //Para el primer año
            for(int edad=0;edad<stockDerivada.length;edad++){
                stockDerSob[0][edad] = stockDerivada[edad];
            }
            
        }
        //Para los demas años
        for(int año=1;año<stockDerSob.length;año++){
            for(int edad=1;edad<stockDerSob[0].length;edad++){
                stockDerSob[año][edad] = (stockDerSob[año-1][edad-1] + derivadasSobSinNuevas[año-1][edad-1]) * probSobDerivadas[año][edad-1];
            }
        }

        this.stockDerivadasSob = stockDerSob;

    }

    public void obtenerTotalDerivadas(double[][] stockSobreviviente,
                                      double[][] derivadasSobSinNuevas,
                                      double[][] derivadasSobFinAnio) {

        double[][] totalCasos = new double[stockSobreviviente.length][stockSobreviviente[0].length];

        for(int año=0;año<totalCasos.length;año++){
            for(int edad=0;edad<totalCasos[0].length;edad++){
                totalCasos[año][edad] = stockSobreviviente[año][edad] + derivadasSobSinNuevas[año][edad] + derivadasSobFinAnio[año][edad];
            }
        }
        this.casosTotalesDerivadas = totalCasos;
    }

    

    public double[][] getPensionadosFallecidos() {
        return pensionadosFallecidos;
    }

    public double[][] getPensionadosSobrevivientes() {
        return pensionadosSobrevivientes;
    }

    public double[][] getNuevasDerivadas() {
        return nuevasDerivadas;
    }

    public double[][] getDerivadasSobFinAnio() {
        return derivadasSobFinAnio;
    }

    public double[][] getDerivadasSobSinNuevas() {
        return derivadasSobSinNuevas;
    }

    public double[][] getStockDerivadasSob() {
        return stockDerivadasSob;
    }

    public double[][] getCasosTotalesDerivadas() {
        return casosTotalesDerivadas;
    }



    

    

    

    

}
